using System;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;
using System.Security.Principal;
using System.Management;

namespace pcNanny
{
	/// <summary>
	/// User information
	/// </summary>
	public class UserInfo
	{
        private const int dataVersion = 2;
        
        private string name = "Anon";
		private int[] timeLimit = new int[7];	// minutes
		private int timeUsed = 0;	// seconds
		private DateTime sessionDate = DateTime.Today;
		private bool enableNag = true;
		private bool enableLog = false;
        private bool screenSaved = false;
        private bool screenLocked = false;

        /// <summary>
        /// User name property
        /// </summary>
		public string Name
		{
			get { return name; }
		}

        /// <summary>
        /// Total time used property
        /// </summary>
		public int TotalTimeUsed
		{
			get { return timeUsed; }
			set { timeUsed = value; }
		}

        /// <summary>
        /// Screen saved property
        /// </summary>
        public bool ScreenSaved
        {
            get { return screenSaved; }
            set { screenSaved = value; }
        }

        /// <summary>
        /// Screen locked property
        /// </summary>
        public bool ScreenLocked
        {
            get { return screenLocked; }
            set { screenLocked = value; }
        }

        /// <summary>
        /// Enable nag property
        /// </summary>
		public bool EnableNag
		{
			get { return enableNag; }
			set { enableNag = value; }
		}

        /// <summary>
        /// Enable log property
        /// </summary>
		public bool EnableLog
		{
			get { return enableLog; }
			set { enableLog = value; }
		}

        /// <summary>
        /// Is user logged in and active?
        /// </summary>
        public bool Active
        {
            get { return name == Environment.UserName &&
                !(screenLocked || screenSaved); }
        }

		/// <summary>
		/// get time limit for specified day
		/// </summary>
		public int GetTimeLimit(DayOfWeek day)
		{
			int dayIndex = (int)day;
			return timeLimit[dayIndex];
		}

		/// <summary>
		/// get time limit for today
		/// </summary>
		public int GetTimeLimit()
		{
			return GetTimeLimit(DateTime.Today.DayOfWeek);
		}

		/// <summary>
		/// set time limit for specified day
		/// </summary>
		public void SetTimeLimit(DayOfWeek day, int limit)
		{
			int dayIndex = (int)day;
			timeLimit[dayIndex] = limit;
		}

		/// <summary>
		/// Get string representation of user
		/// </summary>
		/// <returns>A string containing time "used (limit)" in minutes.</returns>
		public override string ToString()
		{
            return String.Format("{0}:{1:00} ({2}:00)",
                TotalTimeUsed / 60, TotalTimeUsed % 60, GetTimeLimit());
		}

        /// <summary>
        /// Constructor for user
        /// </summary>
        /// <param name="name">name of user</param>
		public UserInfo(string name)
		{
			this.name = name;

			// default time limit until loaded
			for (int i = 0; i < timeLimit.Length; ++i)
			{
				timeLimit[i] = 60; // minutes
			}
		
			Reset();
		}

        /// <summary>
        /// Reset stats for this user.
        /// </summary>
		public void Reset()
		{
			timeUsed = 0;
			sessionDate = DateTime.Today;
		}

        /// <summary>
        /// Add one second to the total time used.
        /// </summary>
		public void AddSecond()
		{
			timeUsed++;
		}

        /// <summary>
        /// Has time limit been exceeded?
        /// </summary>
        /// <returns>true if time limit exceeded</returns>
		public bool IsOverTime()
		{
			return (timeUsed > (GetTimeLimit()*60));
		}

        /// <summary>
        /// Data file property
        /// </summary>
		private string dataFile
		{
			get { return Application.StartupPath + "\\" + Name + ".dat"; }
		}

        /// <summary>
        /// Save user info to DAT file
        /// </summary>
		public void SaveToFile()
		{
			FileStream fs = File.Create(dataFile);
			BinaryWriter bw = new BinaryWriter(fs);

			bw.Write((Int32)dataVersion);
			for (int i = 0; i < timeLimit.Length; ++i)
			{
				bw.Write((Int32)timeLimit[i]);
			}
			bw.Write((Int32)timeUsed);
			bw.Write((Int64)sessionDate.Ticks);
			bw.Write(enableNag);
			bw.Write(enableLog);

			bw.Close();
			fs.Close();
		}

        /// <summary>
        /// Load user info from DAT file
        /// </summary>
		public void LoadFromFile()
		{
			Reset();

			if (File.Exists(dataFile))
			{
				FileStream fs = File.OpenRead(dataFile);
				BinaryReader br = new BinaryReader(fs);

				int fileDataVersion = br.ReadInt32();
				if (fileDataVersion == dataVersion)
				{
					for (int i = 0; i < timeLimit.Length; ++i)
					{
						timeLimit[i] = br.ReadInt32();
					}
					timeUsed = br.ReadInt32();
					sessionDate = new DateTime(br.ReadInt64());
					enableNag = br.ReadBoolean();
					enableLog = br.ReadBoolean();
				}

				br.Close();
				fs.Close();
			}

			// if we load and the date has changed, reset the time used
			if (sessionDate != DateTime.Today)
			{
				Reset();
			}
		}

        /// <summary>
        /// log file property = app-folder/user-name.TXT
        /// </summary>
		private string logFile
		{
			get { return Application.StartupPath + "\\" + Name + ".txt"; }
		}

        /// <summary>
        /// Open the log file for viewing
        /// </summary>
		public void OpenLog()
		{
			System.Diagnostics.Process proc = new System.Diagnostics.Process();
			//proc.EnableRaisingEvents = false;
			proc.StartInfo.FileName = logFile;
			proc.StartInfo.UseShellExecute = true;
			proc.Start();
		}

        /// <summary>
        /// Log a string to the output file
        /// </summary>
        /// <param name="sMsg">message to log</param>
		public void Log(string sMsg)
		{
            sMsg = sMsg.Replace("\n", "\r\n");
            try
            {
                StreamWriter sw = new StreamWriter(logFile, true);	// append
                sw.Write(sMsg);
                sw.Flush();
                sw.Close();
            }
            catch (Exception)
            {
            }
		}

        /// <summary>
        /// Reset log file
        /// </summary>
		public void ResetLog()
		{
            try
            {
                StreamWriter sw = new StreamWriter(logFile, false);	// truncate
                sw.Flush();
                sw.Close();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Log a key press
        /// </summary>
        /// <param name="sender">not used</param>
        /// <param name="e">key press event</param>
		public void KeyPress(object sender, KeyPressEventArgs e)
		{
            if (EnableLog)
            {
                // normalize line endings
                if (e.KeyChar == '\r')
                {
                    Log("\n");
                }
                else
                {
                    Log(e.KeyChar.ToString());
                }
            }
        }

		/// <summary>
		/// check to see if the logged in user is an admin
		/// </summary>
		/// <returns>true if user is an admin</returns>
		static public bool IsAdmin()
		{
			bool isAdmin;
			try
			{
				//get the currently logged in user
				WindowsIdentity user = WindowsIdentity.GetCurrent();
				WindowsPrincipal principal = new WindowsPrincipal(user);
				isAdmin = principal.IsInRole(WindowsBuiltInRole.Administrator);
			}
			catch (Exception)
			{
				isAdmin = false;
			}
			return isAdmin;
		}
	};

	public class UserList : IEnumerable
	{
        private List<UserInfo> list = null;

        /// <summary>
        /// User list constructor
        /// </summary>
        public UserList()
		{
            list = new List<UserInfo>();

            UserInfo me = new UserInfo(Environment.UserName);
            me.LoadFromFile();
            list.Add(me);

            DirectoryInfo di = new DirectoryInfo(Application.StartupPath);
            FileInfo[] files = di.GetFiles("*.dat");
            foreach (FileInfo fi in files)
            {
                string name = Path.GetFileNameWithoutExtension(fi.Name);

				UserInfo found = FindByName(name);
				if (found == null)
				{
                    // Load user
                    UserInfo user = new UserInfo(name);
                    user.LoadFromFile();
                    list.Add(user);
				}
			}

			// save list to file
			foreach (UserInfo user in list)
			{
				user.SaveToFile();
			}

            // test data
            //UserInfo u = new UserInfo("Becky");
            //u.TotalTimeUsed = 33 * 60 + 11;
            //list.Add(u);
            //u = new UserInfo("Cliff");
            //u.TotalTimeUsed = 62 * 60 + 19;
            //list.Add(u);
            //u = new UserInfo("Elizabeth");
            //u.TotalTimeUsed = 22 * 60 + 4;
            //list.Add(u);
            //me.TotalTimeUsed = 22 * 60 + 4;

            // sort list for display
            list.Sort(delegate(UserInfo u1, UserInfo u2)
            { return u1.Name.CompareTo(u2.Name); });
        }

        /// <summary>
        /// Return number of elements in the list
        /// </summary>
        public int Count
        {
            get { return list.Count; }
        }

        /// <summary>
        /// Get the enumerator
        /// </summary>
        /// <returns>enumerator for container</returns>
        public IEnumerator GetEnumerator()
        {
            return list.GetEnumerator();
        }

        /// <summary>
        /// Find user by name
        /// </summary>
        /// <param name="userName">name of user</param>
        /// <returns>user or null</returns>
        public UserInfo FindByName(string userName)
        {
            UserInfo found = list.Find(delegate(UserInfo u)
            {
                return u.Name == userName;
            });
            return found;
        }
    };
}
